@@ -316,8 +316,8 @@ def fd_indefinite_integral(f, t):
 
 def fd_definite_integral(f, t):
     Sfdt = np.zeros_like(f)
-    Sfdt[1:, ...] = (f[1:, ...] + f[:-1, ...]) * ((t[1:] - t[:-1]) / 2.0)
-    return np.sum(Sfdt)
+    Sfdt[1:, ...] = (f[1:, ...] + f[:-1, ...]) * ((t[1:] - t[:-1]) / 2.0).reshape((-1,) + (1,)*(f.ndim-1))
+    return np.sum(Sfdt, axis=0)
 
 
 def spline_evaluation(f, t, t_out=None, axis=None, spline_degree=3,
