@@ -200,14 +200,14 @@ def test_isclose():
 
 @pytest.mark.parametrize("q", make_Qs())
 def test_bad_conversions(q):
-    with pytest.raises(TypeError):
+    with pytest.raises((TypeError, ValueError)):
         s = int(q)
-    with pytest.raises(TypeError):
+    with pytest.raises((TypeError, ValueError)):
         s = float(q)
-    with pytest.raises(TypeError):
+    with pytest.raises((TypeError, ValueError)):
         a = np.zeros(3, dtype=int)
         a[0] = q
-    with pytest.raises(TypeError):
+    with pytest.raises((TypeError, ValueError)):
         a = np.zeros(3)
         a[0] = q
 
@@ -228,6 +228,34 @@ def test_as_float_quat(Qs):
     assert quaternion.as_float_array(quaternion.x).ndim == 1
 
 
+def test_vector_conversions():
+    from quaternion import from_vector_part, as_vector_part
+    v = np.random.rand(3)
+    q = from_vector_part(v, vector_axis=-1)
+    assert q.dtype == np.quaternion
+    assert q.shape == v.shape[:-1]
+    v2 = as_vector_part(q)
+    assert np.array_equal(v, v2)
+    v = np.random.rand(7, 3)
+    q = from_vector_part(v, vector_axis=-1)
+    assert q.dtype == np.quaternion
+    assert q.shape == v.shape[:-1]
+    v2 = as_vector_part(q)
+    assert np.array_equal(v, v2)
+    v = np.random.rand(18, 7, 3)
+    q = from_vector_part(v, vector_axis=-1)
+    assert q.dtype == np.quaternion
+    assert q.shape == v.shape[:-1]
+    v2 = as_vector_part(q)
+    assert np.array_equal(v, v2)
+    v = np.random.rand(18, 3, 7)
+    q = from_vector_part(v, vector_axis=1)
+    assert q.dtype == np.quaternion
+    assert q.shape == v.shape[:1] + v.shape[2:]
+    v2 = as_vector_part(q)
+    assert np.array_equal(v, np.moveaxis(v2, -1, 1))
+
+
 def test_as_rotation_matrix(Rs):
     def quat_mat(quat):
         return np.array([(quat * v * quat.inverse()).vec for v in [quaternion.x, quaternion.y, quaternion.z]]).T
@@ -1075,6 +1103,25 @@ def test_metrics(Rs):
         assert (abs(distance_dict[func] - left_distances) < metric_precision).all()
 
 
+def test_unflip_rotors(Rs):
+    unflip_precision = 4e-16
+    f = 2 * np.random.rand(17, 1_000, 4) - 1
+    q = quaternion.as_quat_array(f)
+    q = q / abs(q)
+    ndim = q.ndim
+    axis = -1
+    inplace = False
+    q_out = quaternion.unflip_rotors(q, axis=axis, inplace=inplace)
+    diff = np.linalg.norm(np.diff(quaternion.as_float_array(q_out), axis=(axis % ndim)), axis=-1)
+    assert np.sum(diff > 1.4142135623730950488016887242097) == 0
+    q_in = np.vstack((Rs, -Rs))
+    q_out = quaternion.unflip_rotors(q_in, axis=0, inplace=False)
+    assert np.array_equal(q_out[1], Rs)
+    q_in = np.vstack((Rs, -Rs))
+    quaternion.unflip_rotors(q_in, axis=0, inplace=True)
+    assert np.array_equal(q_in[1], Rs)
+
+
 def test_slerp(Rs):
     from quaternion import slerp_evaluate, slerp, allclose
     slerp_precision = 4.e-15
